%Copyright (C) 2018 Peter Munro
%
%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU Lesser Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%(at your option) any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU Lesser Public License for more details.
%    
%You should have received a copy of the GNU Lesser Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%set this to 1 to recompute all field quantities
recompute = 0;

%refractive index profile
nvec = [1. 1.54 1.33];
hvec = [-170e-6 0];

%beam parameters
lambda = 488e-9;
NA = 0.9;

%number of values of theta and phi to integrate over
ntheta = 1000;
nphi = 1000;

%this location where the field will evaluated. Note that this lies
%on the optical axis, 50 microns after the last interface.
x = 0;
y = 0;
z = 50e-6;
[X,Y,Z] = ndgrid(x,y,z);
vertices = [X(:) Y(:) Z(:)];

%the PSF is calculated by shifting the statified medium and probe
%vertex axially relative to the focus.
delta_z = linspace(-50e-6,50e-6,501);

%Evaluate the free space beam profile, in the absence of a
%stratified medium
z_fs = linspace(-10e-6,10e-6,501);
%Unaberrated case
if recompute
    [Ep_fs,Em_fs] = focstratfield_general_pol([zeros(numel(z_fs),1) zeros(numel(z_fs),1) z_fs(:)],1,[],NA,lambda,50,50,@xlin_pol);
    save cff08varl32 Ep_fs Em_fs;
else
    load cff08varl32;
end

%Case corresponding with the residual aberration where 4 terms in
%the aberration compensation are considered
%
%xlin_pol_resid uses the residual aberration calculated in
%test_hypothesis.m
if recompute
    [Ep_fs_resid,Em_fs_resid] = focstratfield_general_pol([zeros(numel(z_fs),1) zeros(numel(z_fs),1) z_fs(:)],1.33,[],NA,lambda,50,50,@xlin_pol_resid);
    save cff08varl43 Ep_fs Em_fs;
else
    load cff08varl43;
end

%Used for normalisation
IN = max(sum(abs(Ep_fs).^2,2));

%Nterms is used to control how many Zernike polynomials are used to
%represent the aberration
global Nterms;

%Consider between 3 and 6 Zernike polynomials (inclusive)
Nterms_vec = [3 4 5 6];
fs = 12;
for int = 1:numel(Nterms_vec)
    Nterms = Nterms_vec(int);
    Eacc = zeros(numel(delta_z),3);
    if recompute
	for i=1:numel(delta_z)
	    [Ep,Em] = focstratfield_general_pol(vertices+[0 0 delta_z(i)],nvec,hvec+delta_z(i),NA,lambda,ntheta,nphi,@xlin_pol_comp_07);
	    Eacc(i,:) = Ep(:);
	end
	save(sprintf('cff08vars_%02d',int),'Eacc');
    else
	load(sprintf('cff08vars_%02d',int),'Eacc');
    end
    figure(1);%hold on;
    subplot(3,2,int);
    hp = plot(delta_z*1e6,sum(abs(Eacc).^2,2)/IN);
    
    ht = text(-40,.08,sprintf('%c)','a'+int-1));
    set(ht,'FontSize',16);
    set(ht,'Units','normalized');
    set(ht,'Position',[.02 .8 0]);
    grid on;
    set(hp,'LineWidth',2);
    set(gca,'FontSize',fs);
    
    ax=axis;
    axis([-45 0 ax(3:4)]);
    if int==1
	hx=xlabel('z (\mum)');
	hy = ylabel('I (normalized)');
    end
  
end

subplot(3,2,5);
%generated by calc_focal_field_05.m
if ~exist('cff05vars.mat')
    error('calc_focal_field_05 must be executed before this script can continue');
else
    dat5=load('cff05vars');
end
hp = plot(dat5.delta_z*1e6,sum(abs(dat5.Eacc).^2,2)/IN);
ax = axis;
axis([-45 0 ax(3:4)]);
ht = text(-40,.08,sprintf('%c)','a'+5-1));
set(ht,'FontSize',16);
set(ht,'Units','normalized');
set(ht,'Position',[.02 .8 0]);
grid on;
set(hp,'LineWidth',2);
set(gca,'FontSize',fs);

subplot(3,2,6);
hp = plot(z_fs*1e6,sum(abs(Ep_fs).^2,2)/IN);
ax=axis;
axis([-45/2 45/2 ax(3:4)]);
ht = text(-40,.08,sprintf('%c)','a'+6-1));
set(ht,'FontSize',16);
set(ht,'Units','normalized');
set(ht,'Position',[.02 .8 0]);
grid on;
set(hp,'LineWidth',2);
set(gca,'FontSize',fs);
